/* abstractqueue.c generated by valac 0.46.6, the Vala compiler
 * generated from abstractqueue.vala, do not modify */

/* abstractqueue.vala
 *
 * Copyright (C) 2009  Didier Villevalois
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 * 	Didier 'Ptitjes Villevalois <ptitjes@free.fr>
 */

#include <glib-object.h>
#include <glib.h>

#define GEE_TYPE_TRAVERSABLE (gee_traversable_get_type ())
#define GEE_TRAVERSABLE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_TYPE_TRAVERSABLE, GeeTraversable))
#define GEE_IS_TRAVERSABLE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_TYPE_TRAVERSABLE))
#define GEE_TRAVERSABLE_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GEE_TYPE_TRAVERSABLE, GeeTraversableIface))

typedef struct _GeeTraversable GeeTraversable;
typedef struct _GeeTraversableIface GeeTraversableIface;
typedef gboolean (*GeeForallFunc) (gpointer g, gpointer user_data);
typedef enum  {
	GEE_TRAVERSABLE_STREAM_YIELD,
	GEE_TRAVERSABLE_STREAM_CONTINUE,
	GEE_TRAVERSABLE_STREAM_END,
	GEE_TRAVERSABLE_STREAM_WAIT
} GeeTraversableStream;

#define GEE_TRAVERSABLE_TYPE_STREAM (gee_traversable_stream_get_type ())

#define GEE_TYPE_LAZY (gee_lazy_get_type ())
#define GEE_LAZY(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_TYPE_LAZY, GeeLazy))
#define GEE_LAZY_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GEE_TYPE_LAZY, GeeLazyClass))
#define GEE_IS_LAZY(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_TYPE_LAZY))
#define GEE_IS_LAZY_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GEE_TYPE_LAZY))
#define GEE_LAZY_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GEE_TYPE_LAZY, GeeLazyClass))

typedef struct _GeeLazy GeeLazy;
typedef struct _GeeLazyClass GeeLazyClass;
typedef GeeTraversableStream (*GeeStreamFunc) (GeeTraversableStream state, GeeLazy* g, GeeLazy* * lazy, gpointer user_data);

#define GEE_TYPE_ITERATOR (gee_iterator_get_type ())
#define GEE_ITERATOR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_TYPE_ITERATOR, GeeIterator))
#define GEE_IS_ITERATOR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_TYPE_ITERATOR))
#define GEE_ITERATOR_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GEE_TYPE_ITERATOR, GeeIteratorIface))

typedef struct _GeeIterator GeeIterator;
typedef struct _GeeIteratorIface GeeIteratorIface;
typedef gpointer (*GeeFoldFunc) (gpointer g, gpointer a, gpointer user_data);
typedef gpointer (*GeeMapFunc) (gpointer g, gpointer user_data);
typedef gboolean (*GeePredicate) (gconstpointer g, gpointer user_data);
typedef GeeIterator* (*GeeFlatMapFunc) (gpointer g, gpointer user_data);

#define GEE_TYPE_ITERABLE (gee_iterable_get_type ())
#define GEE_ITERABLE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_TYPE_ITERABLE, GeeIterable))
#define GEE_IS_ITERABLE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_TYPE_ITERABLE))
#define GEE_ITERABLE_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GEE_TYPE_ITERABLE, GeeIterableIface))

typedef struct _GeeIterable GeeIterable;
typedef struct _GeeIterableIface GeeIterableIface;

#define GEE_TYPE_COLLECTION (gee_collection_get_type ())
#define GEE_COLLECTION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_TYPE_COLLECTION, GeeCollection))
#define GEE_IS_COLLECTION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_TYPE_COLLECTION))
#define GEE_COLLECTION_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GEE_TYPE_COLLECTION, GeeCollectionIface))

typedef struct _GeeCollection GeeCollection;
typedef struct _GeeCollectionIface GeeCollectionIface;

#define GEE_TYPE_ABSTRACT_COLLECTION (gee_abstract_collection_get_type ())
#define GEE_ABSTRACT_COLLECTION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_TYPE_ABSTRACT_COLLECTION, GeeAbstractCollection))
#define GEE_ABSTRACT_COLLECTION_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GEE_TYPE_ABSTRACT_COLLECTION, GeeAbstractCollectionClass))
#define GEE_IS_ABSTRACT_COLLECTION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_TYPE_ABSTRACT_COLLECTION))
#define GEE_IS_ABSTRACT_COLLECTION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GEE_TYPE_ABSTRACT_COLLECTION))
#define GEE_ABSTRACT_COLLECTION_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GEE_TYPE_ABSTRACT_COLLECTION, GeeAbstractCollectionClass))

typedef struct _GeeAbstractCollection GeeAbstractCollection;
typedef struct _GeeAbstractCollectionClass GeeAbstractCollectionClass;
typedef struct _GeeAbstractCollectionPrivate GeeAbstractCollectionPrivate;

#define GEE_TYPE_QUEUE (gee_queue_get_type ())
#define GEE_QUEUE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_TYPE_QUEUE, GeeQueue))
#define GEE_IS_QUEUE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_TYPE_QUEUE))
#define GEE_QUEUE_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GEE_TYPE_QUEUE, GeeQueueIface))

typedef struct _GeeQueue GeeQueue;
typedef struct _GeeQueueIface GeeQueueIface;

#define GEE_TYPE_ABSTRACT_QUEUE (gee_abstract_queue_get_type ())
#define GEE_ABSTRACT_QUEUE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_TYPE_ABSTRACT_QUEUE, GeeAbstractQueue))
#define GEE_ABSTRACT_QUEUE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GEE_TYPE_ABSTRACT_QUEUE, GeeAbstractQueueClass))
#define GEE_IS_ABSTRACT_QUEUE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_TYPE_ABSTRACT_QUEUE))
#define GEE_IS_ABSTRACT_QUEUE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GEE_TYPE_ABSTRACT_QUEUE))
#define GEE_ABSTRACT_QUEUE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GEE_TYPE_ABSTRACT_QUEUE, GeeAbstractQueueClass))

typedef struct _GeeAbstractQueue GeeAbstractQueue;
typedef struct _GeeAbstractQueueClass GeeAbstractQueueClass;
typedef struct _GeeAbstractQueuePrivate GeeAbstractQueuePrivate;
enum  {
	GEE_ABSTRACT_QUEUE_0_PROPERTY,
	GEE_ABSTRACT_QUEUE_G_TYPE,
	GEE_ABSTRACT_QUEUE_G_DUP_FUNC,
	GEE_ABSTRACT_QUEUE_G_DESTROY_FUNC,
	GEE_ABSTRACT_QUEUE_CAPACITY_PROPERTY,
	GEE_ABSTRACT_QUEUE_REMAINING_CAPACITY_PROPERTY,
	GEE_ABSTRACT_QUEUE_IS_FULL_PROPERTY,
	GEE_ABSTRACT_QUEUE_NUM_PROPERTIES
};
static GParamSpec* gee_abstract_queue_properties[GEE_ABSTRACT_QUEUE_NUM_PROPERTIES];

struct _GeeIteratorIface {
	GTypeInterface parent_iface;
	gboolean (*next) (GeeIterator* self);
	gboolean (*has_next) (GeeIterator* self);
	gpointer (*get) (GeeIterator* self);
	void (*remove) (GeeIterator* self);
	gboolean (*get_valid) (GeeIterator* self);
	gboolean (*get_read_only) (GeeIterator* self);
};

struct _GeeTraversableIface {
	GTypeInterface parent_iface;
	GType (*get_g_type) (GeeTraversable* self);
	GBoxedCopyFunc (*get_g_dup_func) (GeeTraversable* self);
	GDestroyNotify (*get_g_destroy_func) (GeeTraversable* self);
	gboolean (*foreach) (GeeTraversable* self, GeeForallFunc f, gpointer f_target);
	GeeIterator* (*stream) (GeeTraversable* self, GType a_type, GBoxedCopyFunc a_dup_func, GDestroyNotify a_destroy_func, GeeStreamFunc f, gpointer f_target, GDestroyNotify f_target_destroy_notify);
	gpointer (*fold) (GeeTraversable* self, GType a_type, GBoxedCopyFunc a_dup_func, GDestroyNotify a_destroy_func, GeeFoldFunc f, gpointer f_target, gpointer seed);
	GeeIterator* (*map) (GeeTraversable* self, GType a_type, GBoxedCopyFunc a_dup_func, GDestroyNotify a_destroy_func, GeeMapFunc f, gpointer f_target);
	GeeIterator* (*scan) (GeeTraversable* self, GType a_type, GBoxedCopyFunc a_dup_func, GDestroyNotify a_destroy_func, GeeFoldFunc f, gpointer f_target, gpointer seed);
	GeeIterator* (*filter) (GeeTraversable* self, GeePredicate pred, gpointer pred_target, GDestroyNotify pred_target_destroy_notify);
	GeeIterator* (*chop) (GeeTraversable* self, gint offset, gint length);
	GType (*get_element_type) (GeeTraversable* self);
	GeeIterator* (*flat_map) (GeeTraversable* self, GType a_type, GBoxedCopyFunc a_dup_func, GDestroyNotify a_destroy_func, GeeFlatMapFunc f, gpointer f_target, GDestroyNotify f_target_destroy_notify);
	GeeIterator** (*tee) (GeeTraversable* self, guint forks, gint* result_length1);
	gpointer (*first_match) (GeeTraversable* self, GeePredicate pred, gpointer pred_target, GDestroyNotify pred_target_destroy_notify);
	gboolean (*any_match) (GeeTraversable* self, GeePredicate pred, gpointer pred_target, GDestroyNotify pred_target_destroy_notify);
	gboolean (*all_match) (GeeTraversable* self, GeePredicate pred, gpointer pred_target, GDestroyNotify pred_target_destroy_notify);
	gpointer (*max) (GeeTraversable* self, GCompareDataFunc compare, gpointer compare_target, GDestroyNotify compare_target_destroy_notify);
	gpointer (*min) (GeeTraversable* self, GCompareDataFunc compare, gpointer compare_target, GDestroyNotify compare_target_destroy_notify);
	GeeIterator* (*order_by) (GeeTraversable* self, GCompareDataFunc compare, gpointer compare_target, GDestroyNotify compare_target_destroy_notify);
};

struct _GeeIterableIface {
	GTypeInterface parent_iface;
	GType (*get_g_type) (GeeIterable* self);
	GBoxedCopyFunc (*get_g_dup_func) (GeeIterable* self);
	GDestroyNotify (*get_g_destroy_func) (GeeIterable* self);
	GeeIterator* (*iterator) (GeeIterable* self);
};

struct _GeeCollectionIface {
	GTypeInterface parent_iface;
	GType (*get_g_type) (GeeCollection* self);
	GBoxedCopyFunc (*get_g_dup_func) (GeeCollection* self);
	GDestroyNotify (*get_g_destroy_func) (GeeCollection* self);
	gboolean (*contains) (GeeCollection* self, gconstpointer item);
	gboolean (*add) (GeeCollection* self, gconstpointer item);
	gboolean (*remove) (GeeCollection* self, gconstpointer item);
	void (*clear) (GeeCollection* self);
	gboolean (*add_all) (GeeCollection* self, GeeCollection* collection);
	gboolean (*contains_all) (GeeCollection* self, GeeCollection* collection);
	gboolean (*remove_all) (GeeCollection* self, GeeCollection* collection);
	gboolean (*retain_all) (GeeCollection* self, GeeCollection* collection);
	gpointer* (*to_array) (GeeCollection* self, gint* result_length1);
	gint (*get_size) (GeeCollection* self);
	gboolean (*get_is_empty) (GeeCollection* self);
	gboolean (*get_read_only) (GeeCollection* self);
	GeeCollection* (*get_read_only_view) (GeeCollection* self);
	gboolean (*add_all_array) (GeeCollection* self, gpointer* array, gint array_length1);
	gboolean (*contains_all_array) (GeeCollection* self, gpointer* array, gint array_length1);
	gboolean (*remove_all_array) (GeeCollection* self, gpointer* array, gint array_length1);
	gboolean (*add_all_iterator) (GeeCollection* self, GeeIterator* iter);
	gboolean (*contains_all_iterator) (GeeCollection* self, GeeIterator* iter);
	gboolean (*remove_all_iterator) (GeeCollection* self, GeeIterator* iter);
};

struct _GeeAbstractCollection {
	GObject parent_instance;
	GeeAbstractCollectionPrivate * priv;
};

struct _GeeAbstractCollectionClass {
	GObjectClass parent_class;
	gboolean (*contains) (GeeAbstractCollection* self, gconstpointer item);
	gboolean (*add) (GeeAbstractCollection* self, gconstpointer item);
	gboolean (*remove) (GeeAbstractCollection* self, gconstpointer item);
	void (*clear) (GeeAbstractCollection* self);
	GeeIterator* (*iterator) (GeeAbstractCollection* self);
	gboolean (*foreach) (GeeAbstractCollection* self, GeeForallFunc f, gpointer f_target);
	void (*reserved0) (GeeAbstractCollection* self);
	void (*reserved1) (GeeAbstractCollection* self);
	void (*reserved2) (GeeAbstractCollection* self);
	void (*reserved3) (GeeAbstractCollection* self);
	void (*reserved4) (GeeAbstractCollection* self);
	void (*reserved5) (GeeAbstractCollection* self);
	void (*reserved6) (GeeAbstractCollection* self);
	void (*reserved7) (GeeAbstractCollection* self);
	void (*reserved8) (GeeAbstractCollection* self);
	void (*reserved9) (GeeAbstractCollection* self);
	gint (*get_size) (GeeAbstractCollection* self);
	gboolean (*get_read_only) (GeeAbstractCollection* self);
	GeeCollection* (*get_read_only_view) (GeeAbstractCollection* self);
};

struct _GeeQueueIface {
	GTypeInterface parent_iface;
	GType (*get_g_type) (GeeQueue* self);
	GBoxedCopyFunc (*get_g_dup_func) (GeeQueue* self);
	GDestroyNotify (*get_g_destroy_func) (GeeQueue* self);
	gboolean (*offer) (GeeQueue* self, gconstpointer element);
	gpointer (*peek) (GeeQueue* self);
	gpointer (*poll) (GeeQueue* self);
	gint (*drain) (GeeQueue* self, GeeCollection* recipient, gint amount);
	gint (*get_capacity) (GeeQueue* self);
	gint (*get_remaining_capacity) (GeeQueue* self);
	gboolean (*get_is_full) (GeeQueue* self);
};

struct _GeeAbstractQueue {
	GeeAbstractCollection parent_instance;
	GeeAbstractQueuePrivate * priv;
};

struct _GeeAbstractQueueClass {
	GeeAbstractCollectionClass parent_class;
	gpointer (*peek) (GeeAbstractQueue* self);
	gpointer (*poll) (GeeAbstractQueue* self);
	void (*reserved0) (GeeAbstractQueue* self);
	void (*reserved1) (GeeAbstractQueue* self);
	void (*reserved2) (GeeAbstractQueue* self);
	void (*reserved3) (GeeAbstractQueue* self);
	void (*reserved4) (GeeAbstractQueue* self);
	void (*reserved5) (GeeAbstractQueue* self);
	void (*reserved6) (GeeAbstractQueue* self);
	void (*reserved7) (GeeAbstractQueue* self);
	void (*reserved8) (GeeAbstractQueue* self);
	void (*reserved9) (GeeAbstractQueue* self);
	gint (*get_capacity) (GeeAbstractQueue* self);
	gint (*get_remaining_capacity) (GeeAbstractQueue* self);
	gboolean (*get_is_full) (GeeAbstractQueue* self);
};

struct _GeeAbstractQueuePrivate {
	GType g_type;
	GBoxedCopyFunc g_dup_func;
	GDestroyNotify g_destroy_func;
};

static gint GeeAbstractQueue_private_offset;
static gpointer gee_abstract_queue_parent_class = NULL;
static GeeQueueIface * gee_abstract_queue_gee_queue_parent_iface = NULL;

GType gee_traversable_stream_get_type (void) G_GNUC_CONST;
gpointer gee_lazy_ref (gpointer instance);
void gee_lazy_unref (gpointer instance);
GParamSpec* gee_param_spec_lazy (const gchar* name,
                                 const gchar* nick,
                                 const gchar* blurb,
                                 GType object_type,
                                 GParamFlags flags);
void gee_value_set_lazy (GValue* value,
                         gpointer v_object);
void gee_value_take_lazy (GValue* value,
                          gpointer v_object);
gpointer gee_value_get_lazy (const GValue* value);
GType gee_lazy_get_type (void) G_GNUC_CONST;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (GeeLazy, gee_lazy_unref)
GType gee_iterator_get_type (void) G_GNUC_CONST;
GType gee_traversable_get_type (void) G_GNUC_CONST;
GType gee_iterable_get_type (void) G_GNUC_CONST;
GType gee_collection_get_type (void) G_GNUC_CONST;
GType gee_abstract_collection_get_type (void) G_GNUC_CONST;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (GeeAbstractCollection, g_object_unref)
GType gee_queue_get_type (void) G_GNUC_CONST;
GType gee_abstract_queue_get_type (void) G_GNUC_CONST;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (GeeAbstractQueue, g_object_unref)
gpointer gee_abstract_queue_peek (GeeAbstractQueue* self);
static gpointer gee_abstract_queue_real_peek (GeeAbstractQueue* self);
gpointer gee_abstract_queue_poll (GeeAbstractQueue* self);
static gpointer gee_abstract_queue_real_poll (GeeAbstractQueue* self);
G_GNUC_INTERNAL void gee_abstract_queue_reserved0 (GeeAbstractQueue* self);
static void gee_abstract_queue_real_reserved0 (GeeAbstractQueue* self);
G_GNUC_INTERNAL void gee_abstract_queue_reserved1 (GeeAbstractQueue* self);
static void gee_abstract_queue_real_reserved1 (GeeAbstractQueue* self);
G_GNUC_INTERNAL void gee_abstract_queue_reserved2 (GeeAbstractQueue* self);
static void gee_abstract_queue_real_reserved2 (GeeAbstractQueue* self);
G_GNUC_INTERNAL void gee_abstract_queue_reserved3 (GeeAbstractQueue* self);
static void gee_abstract_queue_real_reserved3 (GeeAbstractQueue* self);
G_GNUC_INTERNAL void gee_abstract_queue_reserved4 (GeeAbstractQueue* self);
static void gee_abstract_queue_real_reserved4 (GeeAbstractQueue* self);
G_GNUC_INTERNAL void gee_abstract_queue_reserved5 (GeeAbstractQueue* self);
static void gee_abstract_queue_real_reserved5 (GeeAbstractQueue* self);
G_GNUC_INTERNAL void gee_abstract_queue_reserved6 (GeeAbstractQueue* self);
static void gee_abstract_queue_real_reserved6 (GeeAbstractQueue* self);
G_GNUC_INTERNAL void gee_abstract_queue_reserved7 (GeeAbstractQueue* self);
static void gee_abstract_queue_real_reserved7 (GeeAbstractQueue* self);
G_GNUC_INTERNAL void gee_abstract_queue_reserved8 (GeeAbstractQueue* self);
static void gee_abstract_queue_real_reserved8 (GeeAbstractQueue* self);
G_GNUC_INTERNAL void gee_abstract_queue_reserved9 (GeeAbstractQueue* self);
static void gee_abstract_queue_real_reserved9 (GeeAbstractQueue* self);
GeeAbstractQueue* gee_abstract_queue_construct (GType object_type,
                                                GType g_type,
                                                GBoxedCopyFunc g_dup_func,
                                                GDestroyNotify g_destroy_func);
GeeAbstractCollection* gee_abstract_collection_construct (GType object_type,
                                                          GType g_type,
                                                          GBoxedCopyFunc g_dup_func,
                                                          GDestroyNotify g_destroy_func);
gint gee_abstract_queue_get_capacity (GeeAbstractQueue* self);
gint gee_abstract_queue_get_remaining_capacity (GeeAbstractQueue* self);
gboolean gee_abstract_queue_get_is_full (GeeAbstractQueue* self);
static void _vala_gee_abstract_queue_get_property (GObject * object,
                                            guint property_id,
                                            GValue * value,
                                            GParamSpec * pspec);
static void _vala_gee_abstract_queue_set_property (GObject * object,
                                            guint property_id,
                                            const GValue * value,
                                            GParamSpec * pspec);

static inline gpointer
gee_abstract_queue_get_instance_private (GeeAbstractQueue* self)
{
	return G_STRUCT_MEMBER_P (self, GeeAbstractQueue_private_offset);
}

/**
 * {@inheritDoc}
 */
static gpointer
gee_abstract_queue_real_peek (GeeAbstractQueue* self)
{
	g_critical ("Type `%s' does not implement abstract method `gee_abstract_queue_peek'", g_type_name (G_TYPE_FROM_INSTANCE (self)));
	return NULL;
}

gpointer
gee_abstract_queue_peek (GeeAbstractQueue* self)
{
	g_return_val_if_fail (self != NULL, NULL);
	return GEE_ABSTRACT_QUEUE_GET_CLASS (self)->peek (self);
}

/**
 * {@inheritDoc}
 */
static gpointer
gee_abstract_queue_real_poll (GeeAbstractQueue* self)
{
	g_critical ("Type `%s' does not implement abstract method `gee_abstract_queue_poll'", g_type_name (G_TYPE_FROM_INSTANCE (self)));
	return NULL;
}

gpointer
gee_abstract_queue_poll (GeeAbstractQueue* self)
{
	g_return_val_if_fail (self != NULL, NULL);
	return GEE_ABSTRACT_QUEUE_GET_CLASS (self)->poll (self);
}

static void
gee_abstract_queue_real_reserved0 (GeeAbstractQueue* self)
{
}

void
gee_abstract_queue_reserved0 (GeeAbstractQueue* self)
{
	g_return_if_fail (self != NULL);
	GEE_ABSTRACT_QUEUE_GET_CLASS (self)->reserved0 (self);
}

static void
gee_abstract_queue_real_reserved1 (GeeAbstractQueue* self)
{
}

void
gee_abstract_queue_reserved1 (GeeAbstractQueue* self)
{
	g_return_if_fail (self != NULL);
	GEE_ABSTRACT_QUEUE_GET_CLASS (self)->reserved1 (self);
}

static void
gee_abstract_queue_real_reserved2 (GeeAbstractQueue* self)
{
}

void
gee_abstract_queue_reserved2 (GeeAbstractQueue* self)
{
	g_return_if_fail (self != NULL);
	GEE_ABSTRACT_QUEUE_GET_CLASS (self)->reserved2 (self);
}

static void
gee_abstract_queue_real_reserved3 (GeeAbstractQueue* self)
{
}

void
gee_abstract_queue_reserved3 (GeeAbstractQueue* self)
{
	g_return_if_fail (self != NULL);
	GEE_ABSTRACT_QUEUE_GET_CLASS (self)->reserved3 (self);
}

static void
gee_abstract_queue_real_reserved4 (GeeAbstractQueue* self)
{
}

void
gee_abstract_queue_reserved4 (GeeAbstractQueue* self)
{
	g_return_if_fail (self != NULL);
	GEE_ABSTRACT_QUEUE_GET_CLASS (self)->reserved4 (self);
}

static void
gee_abstract_queue_real_reserved5 (GeeAbstractQueue* self)
{
}

void
gee_abstract_queue_reserved5 (GeeAbstractQueue* self)
{
	g_return_if_fail (self != NULL);
	GEE_ABSTRACT_QUEUE_GET_CLASS (self)->reserved5 (self);
}

static void
gee_abstract_queue_real_reserved6 (GeeAbstractQueue* self)
{
}

void
gee_abstract_queue_reserved6 (GeeAbstractQueue* self)
{
	g_return_if_fail (self != NULL);
	GEE_ABSTRACT_QUEUE_GET_CLASS (self)->reserved6 (self);
}

static void
gee_abstract_queue_real_reserved7 (GeeAbstractQueue* self)
{
}

void
gee_abstract_queue_reserved7 (GeeAbstractQueue* self)
{
	g_return_if_fail (self != NULL);
	GEE_ABSTRACT_QUEUE_GET_CLASS (self)->reserved7 (self);
}

static void
gee_abstract_queue_real_reserved8 (GeeAbstractQueue* self)
{
}

void
gee_abstract_queue_reserved8 (GeeAbstractQueue* self)
{
	g_return_if_fail (self != NULL);
	GEE_ABSTRACT_QUEUE_GET_CLASS (self)->reserved8 (self);
}

static void
gee_abstract_queue_real_reserved9 (GeeAbstractQueue* self)
{
}

void
gee_abstract_queue_reserved9 (GeeAbstractQueue* self)
{
	g_return_if_fail (self != NULL);
	GEE_ABSTRACT_QUEUE_GET_CLASS (self)->reserved9 (self);
}

GeeAbstractQueue*
gee_abstract_queue_construct (GType object_type,
                              GType g_type,
                              GBoxedCopyFunc g_dup_func,
                              GDestroyNotify g_destroy_func)
{
	GeeAbstractQueue * self = NULL;
	self = (GeeAbstractQueue*) gee_abstract_collection_construct (object_type, g_type, (GBoxedCopyFunc) g_dup_func, (GDestroyNotify) g_destroy_func);
	self->priv->g_type = g_type;
	self->priv->g_dup_func = g_dup_func;
	self->priv->g_destroy_func = g_destroy_func;
	return self;
}

gint
gee_abstract_queue_get_capacity (GeeAbstractQueue* self)
{
	g_return_val_if_fail (self != NULL, 0);
	return GEE_ABSTRACT_QUEUE_GET_CLASS (self)->get_capacity (self);
}

gint
gee_abstract_queue_get_remaining_capacity (GeeAbstractQueue* self)
{
	g_return_val_if_fail (self != NULL, 0);
	return GEE_ABSTRACT_QUEUE_GET_CLASS (self)->get_remaining_capacity (self);
}

gboolean
gee_abstract_queue_get_is_full (GeeAbstractQueue* self)
{
	g_return_val_if_fail (self != NULL, FALSE);
	return GEE_ABSTRACT_QUEUE_GET_CLASS (self)->get_is_full (self);
}

static void
gee_abstract_queue_class_init (GeeAbstractQueueClass * klass,
                               gpointer klass_data)
{
	gee_abstract_queue_parent_class = g_type_class_peek_parent (klass);
	g_type_class_adjust_private_offset (klass, &GeeAbstractQueue_private_offset);
	((GeeAbstractQueueClass *) klass)->peek = (gpointer (*) (GeeAbstractQueue*)) gee_abstract_queue_real_peek;
	((GeeAbstractQueueClass *) klass)->poll = (gpointer (*) (GeeAbstractQueue*)) gee_abstract_queue_real_poll;
	((GeeAbstractQueueClass *) klass)->reserved0 = (void (*) (GeeAbstractQueue*)) gee_abstract_queue_real_reserved0;
	((GeeAbstractQueueClass *) klass)->reserved1 = (void (*) (GeeAbstractQueue*)) gee_abstract_queue_real_reserved1;
	((GeeAbstractQueueClass *) klass)->reserved2 = (void (*) (GeeAbstractQueue*)) gee_abstract_queue_real_reserved2;
	((GeeAbstractQueueClass *) klass)->reserved3 = (void (*) (GeeAbstractQueue*)) gee_abstract_queue_real_reserved3;
	((GeeAbstractQueueClass *) klass)->reserved4 = (void (*) (GeeAbstractQueue*)) gee_abstract_queue_real_reserved4;
	((GeeAbstractQueueClass *) klass)->reserved5 = (void (*) (GeeAbstractQueue*)) gee_abstract_queue_real_reserved5;
	((GeeAbstractQueueClass *) klass)->reserved6 = (void (*) (GeeAbstractQueue*)) gee_abstract_queue_real_reserved6;
	((GeeAbstractQueueClass *) klass)->reserved7 = (void (*) (GeeAbstractQueue*)) gee_abstract_queue_real_reserved7;
	((GeeAbstractQueueClass *) klass)->reserved8 = (void (*) (GeeAbstractQueue*)) gee_abstract_queue_real_reserved8;
	((GeeAbstractQueueClass *) klass)->reserved9 = (void (*) (GeeAbstractQueue*)) gee_abstract_queue_real_reserved9;
	G_OBJECT_CLASS (klass)->get_property = _vala_gee_abstract_queue_get_property;
	G_OBJECT_CLASS (klass)->set_property = _vala_gee_abstract_queue_set_property;
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_ABSTRACT_QUEUE_G_TYPE, g_param_spec_gtype ("g-type", "type", "type", G_TYPE_NONE, G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_ABSTRACT_QUEUE_G_DUP_FUNC, g_param_spec_pointer ("g-dup-func", "dup func", "dup func", G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_ABSTRACT_QUEUE_G_DESTROY_FUNC, g_param_spec_pointer ("g-destroy-func", "destroy func", "destroy func", G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
	/**
	 * {@inheritDoc}
	 */
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_ABSTRACT_QUEUE_CAPACITY_PROPERTY, gee_abstract_queue_properties[GEE_ABSTRACT_QUEUE_CAPACITY_PROPERTY] = g_param_spec_int ("capacity", "capacity", "capacity", G_MININT, G_MAXINT, 0, G_PARAM_STATIC_STRINGS | G_PARAM_READABLE));
	/**
	 * {@inheritDoc}
	 */
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_ABSTRACT_QUEUE_REMAINING_CAPACITY_PROPERTY, gee_abstract_queue_properties[GEE_ABSTRACT_QUEUE_REMAINING_CAPACITY_PROPERTY] = g_param_spec_int ("remaining-capacity", "remaining-capacity", "remaining-capacity", G_MININT, G_MAXINT, 0, G_PARAM_STATIC_STRINGS | G_PARAM_READABLE));
	/**
	 * {@inheritDoc}
	 */
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_ABSTRACT_QUEUE_IS_FULL_PROPERTY, gee_abstract_queue_properties[GEE_ABSTRACT_QUEUE_IS_FULL_PROPERTY] = g_param_spec_boolean ("is-full", "is-full", "is-full", FALSE, G_PARAM_STATIC_STRINGS | G_PARAM_READABLE));
}

static GType
gee_abstract_queue_gee_queue_get_g_type (GeeAbstractQueue* self)
{
	return self->priv->g_type;
}

static GBoxedCopyFunc
gee_abstract_queue_gee_queue_get_g_dup_func (GeeAbstractQueue* self)
{
	return self->priv->g_dup_func;
}

static GDestroyNotify
gee_abstract_queue_gee_queue_get_g_destroy_func (GeeAbstractQueue* self)
{
	return self->priv->g_destroy_func;
}

static void
gee_abstract_queue_gee_queue_interface_init (GeeQueueIface * iface,
                                             gpointer iface_data)
{
	gee_abstract_queue_gee_queue_parent_iface = g_type_interface_peek_parent (iface);
	iface->peek = (gpointer (*) (GeeQueue*)) gee_abstract_queue_peek;
	iface->poll = (gpointer (*) (GeeQueue*)) gee_abstract_queue_poll;
	iface->get_g_type = (GType (*) (GeeQueue *)) gee_abstract_queue_gee_queue_get_g_type;
	iface->get_g_dup_func = (GBoxedCopyFunc (*) (GeeQueue *)) gee_abstract_queue_gee_queue_get_g_dup_func;
	iface->get_g_destroy_func = (GDestroyNotify (*) (GeeQueue *)) gee_abstract_queue_gee_queue_get_g_destroy_func;
	iface->get_capacity = (gint (*) (GeeQueue *)) gee_abstract_queue_get_capacity;
	iface->get_remaining_capacity = (gint (*) (GeeQueue *)) gee_abstract_queue_get_remaining_capacity;
	iface->get_is_full = (gboolean (*) (GeeQueue *)) gee_abstract_queue_get_is_full;
}

static void
gee_abstract_queue_instance_init (GeeAbstractQueue * self,
                                  gpointer klass)
{
	self->priv = gee_abstract_queue_get_instance_private (self);
}

/**
 * Skeletal implementation of the {@link Queue} interface.
 *
 * Contains common code shared by all queue implementations.
 *
 * @see PriorityQueue
 */
GType
gee_abstract_queue_get_type (void)
{
	static volatile gsize gee_abstract_queue_type_id__volatile = 0;
	if (g_once_init_enter (&gee_abstract_queue_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (GeeAbstractQueueClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) gee_abstract_queue_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (GeeAbstractQueue), 0, (GInstanceInitFunc) gee_abstract_queue_instance_init, NULL };
		static const GInterfaceInfo gee_queue_info = { (GInterfaceInitFunc) gee_abstract_queue_gee_queue_interface_init, (GInterfaceFinalizeFunc) NULL, NULL};
		GType gee_abstract_queue_type_id;
		gee_abstract_queue_type_id = g_type_register_static (GEE_TYPE_ABSTRACT_COLLECTION, "GeeAbstractQueue", &g_define_type_info, G_TYPE_FLAG_ABSTRACT);
		g_type_add_interface_static (gee_abstract_queue_type_id, GEE_TYPE_QUEUE, &gee_queue_info);
		GeeAbstractQueue_private_offset = g_type_add_instance_private (gee_abstract_queue_type_id, sizeof (GeeAbstractQueuePrivate));
		g_once_init_leave (&gee_abstract_queue_type_id__volatile, gee_abstract_queue_type_id);
	}
	return gee_abstract_queue_type_id__volatile;
}

static void
_vala_gee_abstract_queue_get_property (GObject * object,
                                       guint property_id,
                                       GValue * value,
                                       GParamSpec * pspec)
{
	GeeAbstractQueue * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, GEE_TYPE_ABSTRACT_QUEUE, GeeAbstractQueue);
	switch (property_id) {
		case GEE_ABSTRACT_QUEUE_G_TYPE:
		g_value_set_gtype (value, self->priv->g_type);
		break;
		case GEE_ABSTRACT_QUEUE_G_DUP_FUNC:
		g_value_set_pointer (value, self->priv->g_dup_func);
		break;
		case GEE_ABSTRACT_QUEUE_G_DESTROY_FUNC:
		g_value_set_pointer (value, self->priv->g_destroy_func);
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}

static void
_vala_gee_abstract_queue_set_property (GObject * object,
                                       guint property_id,
                                       const GValue * value,
                                       GParamSpec * pspec)
{
	GeeAbstractQueue * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, GEE_TYPE_ABSTRACT_QUEUE, GeeAbstractQueue);
	switch (property_id) {
		case GEE_ABSTRACT_QUEUE_G_TYPE:
		self->priv->g_type = g_value_get_gtype (value);
		break;
		case GEE_ABSTRACT_QUEUE_G_DUP_FUNC:
		self->priv->g_dup_func = g_value_get_pointer (value);
		break;
		case GEE_ABSTRACT_QUEUE_G_DESTROY_FUNC:
		self->priv->g_destroy_func = g_value_get_pointer (value);
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}

